part of '/masamune.dart';

/// Create an extension method for [Ref] to handle Query for controllers and controller groups.
///
/// コントローラーやコントローラーグループ用のQueryを処理するための[Ref]の拡張メソッドを作成します。
extension MasamuneControllerAppRefExtensions on AppRef {
  /// A [TController] whose state is retained is obtained by passing the [ControllerQueryBase] generated by the builder.
  ///
  /// ビルダーによりコード生成された[ControllerQueryBase]を渡すことにより状態を保持された[TController]を取得します。
  ///
  /// ```dart
  /// final userController = appRef.controller(UserController.query()); // Get the user controller.
  /// final userControllerGroup = appRef.controller(UserControllerGroup.query()); // Get the user controller group.
  /// ```
  TController controller<TController extends Listenable>(
    ControllerQueryBase<TController> query,
  ) {
    return this.query(query);
  }
}

/// Create an extension method for [RefHasApp] to handle Query for controllers and controller groups.
///
/// コントローラーやコントローラーグループ用のQueryを処理するための[RefHasApp]の拡張メソッドを作成します。
extension MasamuneControllerRefHasAppExtensions on RefHasApp {
  /// A [TController] whose state is retained is obtained by passing the [ControllerQueryBase] generated by the builder.
  ///
  /// The state is always saved in the application scope and `autoDisposeWhenUnreferenced` is set to `true`. Thus, if the referenced widget is no longer available, it will be automatically disposed of.
  ///
  /// ビルダーによりコード生成された[ControllerQueryBase]を渡すことにより状態を保持された[TController]を取得します。
  ///
  /// かならずアプリケーションスコープで状態が保存され`autoDisposeWhenUnreferenced`が`true`になります。そのため、参照されるウィジェットがなくなった場合、自動的に破棄されます。
  ///
  /// ```dart
  /// final userController = ref.controller(UserController.query()); // Get the user controller.
  /// final userControllerGroup = ref.controller(UserControllerGroup.query()); // Get the user controller group.
  /// ```
  TController controller<TController extends Listenable>(
    ControllerQueryBase<TController> query,
  ) {
    // ignore: invalid_use_of_protected_member
    return app.query(query);
  }
}

/// Create extension methods for [PageRef] and [WidgetRef] in the **Application Scope** to handle Query for controllers and controller groups.
///
/// コントローラーやコントローラーグループ用のQueryを処理するための**アプリケーションスコープ**の[PageRef]や[WidgetRef]の拡張メソッドを作成します。
extension MasamuneControllerAppScopedValueRefExtensions on AppScopedValueRef {
  @Deprecated(
    "[controller] with App scope is no longer available. Please use [ref.controller] instead. Only the App scope will continue to be used. Appスコープを指定しての[controller]は利用できなくなります。代わりに[ref.controller]を利用してください。引き続きAppスコープのみでの利用となります。",
  )
  TController controller<TController extends Listenable>(
    ControllerQueryBase<TController> query,
  ) {
    return this.query(query);
  }
}

/// Create extension methods for [PageRef] and [WidgetRef] in **Page Scope** to handle Query for controllers and controller groups.
///
/// コントローラーやコントローラーグループ用のQueryを処理するための**ページスコープ**の[PageRef]や[WidgetRef]の拡張メソッドを作成します。
extension MasamuneControllerPageScopedValueRefExtensions on PageScopedValueRef {
  @Deprecated(
    "[controller] with page scope is no longer available. Please use [ref.controller] instead. ページスコープを指定しての[controller]は利用できなくなります。代わりに[ref.controller]を利用してください。Appスコープのみでの利用となります。",
  )
  TController controller<TController extends Listenable>(
    ControllerQueryBase<TController> query,
  ) {
    return this.query(query);
  }
}

/// Base class for creating state-to-state usage queries for controllers that are code-generated by the builder.
///
/// Basically, you can get a class that inherits from [ChangeNotifier].
///
/// ビルダーによりコード生成するコントローラーの状態間利用クエリを作成するためのベースクラス。
///
/// 基本的には[ChangeNotifier]を継承したクラスを取得することが出来ます。
abstract class ControllerQueryBase<TController extends Listenable>
    extends ChangeNotifierScopedQuery<TController> {
  /// Base class for creating state-to-state usage queries for controllers that are code-generated by the builder.
  ///
  /// Basically, you can get a class that inherits from [ChangeNotifier].
  ///
  /// ビルダーによりコード生成するコントローラーの状態間利用クエリを作成するためのベースクラス。
  ///
  /// 基本的には[ChangeNotifier]を継承したクラスを取得することが出来ます。
  const ControllerQueryBase()
      : super(
          _provider,
        );

  static TController _provider<TController extends Listenable>(Ref ref) {
    throw UnimplementedError();
  }

  @override
  bool get autoDisposeWhenUnreferenced => true;
}
